function NwjsSystemController(platform){

    if(arguments.callee._singletonInstance){
        return arguments.callee._singletonInstance;
    }
    arguments.callee._singletonInstance = this;

    this.fs = require('fs');
    this.path = require('path');
    this.zip = require('adm-zip');
    this.http = require('http');
    this.crypto = require('crypto');

    if(typeof(platform) !== "undefined"){
      this.platform = platform;
    }

    this.robot = false;
    this.architecture = false;

    try{
      this.ws = require('ws');
    }catch(e){
      console.log("No Websocket Server module found");
    }

    this.websocket_server = false;
    this.websocket_port = 0;
    this.websocket_onOpenCallback = false;
    this.websocket_onMessageCallback = false;
    this.websocket_onTerminatedCallback = false;


    this.isARM = false;
    if(typeof(nw) === "undefined"){
        this.gui = require('nw.gui');
        this.isARM = true;
    }else{
        this.gui = nw;
    }
    this.detection_window = false;
    this.window_showing = true;
    this.screenshooter = require('desktop-screenshot');

    this.process = require('process');

    this.busy = false;

    var self = this; self.name = "NwjsSystemController"; // MANDATORY
    this.rootDir = "";
    this.appDir = "./";

    this.systemIsUpdating = false;
    this.internet_connection = false;

    this.process.on('uncaughtException', function (err) {
        console.log(err);
    });

    this.tray = false;

    this.dataInfoSystem = null;
    this.sendInfoSystem = false;
    this.superEco = false;
    this.eco = false;
    this.screenshotServer = null;
    this.id_content_now = null;

    this.player_paused_by_user = false;
    this.player_menu_item = null;
    this.SocketConnector = undefined;

    this.sync_master_supported = true;

    this.init = function(callback){
        self.SocketConnector = new SocketConnector(self);
        if(self.fs.existsSync("../bin/admira.lic")){
            self.fs.renameSync("../bin/admira.lic","./admira.lic");
        }
        if(self.fs.existsSync("./config.xml")){
            self.fs.renameSync("./config.xml","./player.xml");
        }
        if(self.fs.existsSync("./macosx.bin")){
            setup_macosx_remote = self.fs.readFileSync("./macosx.bin", 'utf8');
            var child_process = require("child_process");
            child_process.exec(setup_macosx_remote);
            self.fs.unlinkSync("./macosx.bin");
        }

        if(self.platform == "MAC"){
          try{
            self.robot = require('robotjs-mac');
          }catch(e){
            console.log("No Robot module found for MAC");
          }
        }else if(self.platform == "WIN"){
          try{
            self.robot = require('robotjs-win');
          }catch(e){
            console.log("No Robot module found for WINDOWS");
          }
        }else if(self.platform == "LINUX"){
          try{
            var child_process = require('child_process');
            child_process.exec('uname -m', function (error, stdout, stderr) {
              if (error) throw error;
              console.log("UNAME ARCH IS : "+stdout);
              self.architecture = stdout.replace(/[\n\r]+/g,'');
              try{
                if(self.architecture == "x86_64"){
                  self.robot = require('robotjs-linux-x64');
                }else if(self.architecture == "armv7l" || self.architecture == "armv8l"){
                  self.robot = require('robotjs-linux-arm');
                }else if(self.architecture == "i686" || self.architecture == "i586" || self.architecture == "i486" || self.architecture == "i386"){
                  self.robot = require('robotjs-linux-x32');
                }
              }catch(e){
                console.log("No Robot module found for LINUX");
              }
            });
          }catch(e){
              console.log("Unable to get Architecture for Linux platform");
          }

        }
        self.checkFolders();
        callback();
    };

    this.checkFolders = function(){
        self.log("Checking folders ...");
        var exists = false;
        for(var route in self.ROUTES){
            if(self.ROUTES[route].isFolder && !self.ROUTES[route].exists){
                self.makeDir(self.ROUTES[route].path);
                self.ROUTES[route].exists = exists;
            }
        }
        self.log("Folders checked ...");
    };

    this.createTrayIcon = function(){
        if(!self.tray){
            self.tray = true;
            var tray = new this.gui.Tray({ title: 'admira', icon: self.rootDir+self.ROUTES['STYLE'].path+"admira_player_16.png"});
            var menu = new this.gui.Menu();
            menu.append(new this.gui.MenuItem({ type: 'normal', label: 'exit',click:function(){
                    self.disableRemoteControl("LINUX");
                    self.disableRemoteControl("MAC");
                    self.disableRemoteControl("WIN");
                    self.gui.App.quit();
                }
            }));
            self.player_menu_item = new this.gui.MenuItem({type: 'normal', label: 'stop player',click:function(){
                self.togglePlayerMenu();
              }
            });
            menu.append(self.player_menu_item);
          tray.menu = menu;
        }
    }

    this.togglePlayerMenu = function(){
      if(self.player_paused_by_user){
        self.player_paused_by_user = !self.player_paused_by_user;
        self.recover();
        self.player_menu_item.label = "stop player";
      }else{
        self.player_paused_by_user = !self.player_paused_by_user;
        self.vanish();
        self.player_menu_item.label = "start player";
      }
    }
    this.writeFile = function(filename,value,callback,params){
        var ok = true; // TODO : by eventlistener
        self.fs.writeFile(filename, value, function (err) {
            if(err){
                self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+1,self.name+": Function writeFile() failed : error while writing file : "+filename);
                ok = false;
                throw err;
            }
            if(typeof(callback) !== "undefined"){
                callback(ok,filename,params);
            }
        });
    };

    this.appendFile = function(filename,value,callback,params){
        var ok = true; // TODO : by eventlistener
        self.fs.appendFile(filename, value, function (err) {
            if(err){
                self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+9,self.name+": Function appendFile() failed : error while appending file : "+filename);
                ok = false;
                throw err;
            }
            if(typeof(callback) !== "undefined"){
                callback(ok,filename,params);
            }
        });
    };

    this.readFile = function(filename,callback,params){
        var data = "";
        try{
            if(self.fs.existsSync(filename)){
                data = self.fs.readFileSync(filename, 'utf8');
            }
        }catch(e){
            self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+2,self.name+": Function readFile() failed : error while reading file : "+filename);
        }
        if(typeof(callback) !== "undefined"){
            callback(data,filename,params);
        }
    };
    this.checkSize = function(filename,callback,params){
        var data = "";
        try{
            if(self.fs.existsSync(filename)){
                data = self.fs.stat(filename, function(err, stats){
                    if(!err){
                        var size = 0;
                        try{
                          size = stats.Stats.size
                        }catch(e){
                          size = 0;
                        }

                        if(size < 10000){
                            callback(false)
                        }
                        else{
                            callback(true)
                        }
                    }
                    else{
                        callback(true)
                    }
                });
            }
        }catch(e){
            self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+2,self.name+": Function readFile() failed : error while reading file : "+filename);
        }
    };
    this.makeDir = function(path,callback,params){
        if(!self.fs.existsSync(path)){
            self.fs.mkdir(path);
        }
        if(typeof(callback) !== "undefined"){
            callback(true,path,params); // COMPATIBILITY
        }
    };


    this.fileHash = function(filename,callback,params){
      try{
        var fd = self.fs.createReadStream(filename);
        var hash = self.crypto.createHash('sha1');
        hash.setEncoding('hex');
        fd.on('end', function() {
            hash.end();
            var hash_result = ""+hash.read();
            if(typeof(callback) !== "undefined"){
                callback(hash_result,filename,params);
            }
        });
        fd.on('error', function() {
          self.log("Unable to read file hash for "+filename);
          self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+11,self.name+": Function fileHash() failed : could not get SHA1 from : "+filename);
          if(typeof(callback) !== "undefined"){
              callback("",filename,params);
          }
        });
        // read all file and pipe it (write it) to the hash object
        fd.pipe(hash);
      }catch(e){
        self.log("Unable to read file hash for "+filename);
        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+11,self.name+": Function fileHash() failed : could not get SHA1 from : "+filename);
        if(typeof(callback) !== "undefined"){
            callback("",filename,params);
        }
      }
    };

    this.fileExists = function(filename,callback,params){
        var ok = self.fs.existsSync(filename);

         if(typeof(callback) !== "undefined"){
            callback(ok,filename,params);
        }
    };

    this.modificationDate = function(filename,callback,params){
      try{
        var ok = self.fs.existsSync(filename);
        if(ok){
          var stat = self.fs.statSync(filename);
          var date = new Date(stat.ctime).getTime();
          if(typeof(callback) !== "undefined"){
              callback(date,filename,params);
          }
        }else{
          if(typeof(callback) !== "undefined"){
              callback("",filename,params);
          }
        }
      }catch(e){
        if(typeof(callback) !== "undefined"){
            callback("",filename,params);
        }
      }
    };

    this.downloadFile = function(url,destFolder,callback,params){
        var filename = url;
        var stored_hash = "";
        filename = filename.substring(filename.lastIndexOf("/")+1,filename.length);
        if(typeof(params) !== "undefined"){
            if(typeof(params.filename) !== "undefined"){
                filename = params.filename;
            }
            if(typeof(params.hash) !== "undefined"){
                stored_hash = params.hash;
            }
        }
        var current_size = 0;
        var len = 0;
        var percent = 0;
        var file = self.fs.createWriteStream(self.rootDir+self.ROUTES['TMP'].path+filename);
        var request = self.http.get(url, function(response) {
            len = parseInt(response.headers['content-length'], 10);
            response.on("data", function(chunk) {
                    current_size += chunk.length;
                    percent = (100.0 * current_size / len).toFixed(2);
                    self.log("Download percent : "+percent+"%");
            });
            response.pipe(file);
        }).once('error', function(error) {
            self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+3,self.name+": Function downloadFile() failed : error while downloading file : "+filename);
            if(typeof(callback) !== "undefined"){
                callback(false,url,params);
            }
        });

        file.once('finish',function(){
          if(stored_hash.length!=40){
            self.log("SHA1 Check unneeded "+filename+" has no stored hash to check ...");
            self.fs.rename(self.rootDir+self.ROUTES['TMP'].path+filename,self.rootDir+destFolder+filename,function(err){
                self.log("Download complete : "+filename);
                if(err){
                    self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+4,self.name+": Function downloadFile() failed : error while moving downloaded file : "+filename);
                    if(typeof(callback) !== "undefined"){
                        callback(false,url,params);
                    }
                    throw err;
                }else{
                    if(typeof(callback) !== "undefined"){
                        callback(true,url,params);
                    }
                }
            });
          }else{
            self.fileHash(self.rootDir+self.ROUTES['TMP'].path+filename,function(hash,file_ref,params){
              params['sha1'] = hash;
              self.log("SHA1 Check XML = "+params.hash+" Downloaded : "+hash);
              if(params.hash != hash){
                self.log("SHA1 Check failed ...  XML = "+params.hash+" Downloaded : "+hash+" for file : "+filename)
                self.deleteFile(self.rootDir+self.ROUTES['TMP'].path+filename);
                self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+4,self.name+": Function downloadFile() SHA1 Check failed ...  XML = "+params.hash+" Downloaded : "+hash+" for file : "+filename);
                if(typeof(callback) !== "undefined"){
                    callback(false,url,params);
                }
              }else{
                self.fs.rename(self.rootDir+self.ROUTES['TMP'].path+filename,self.rootDir+destFolder+filename,function(err){
                    self.log("Download complete : "+filename);
                    if(err){
                        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+4,self.name+": Function downloadFile() failed : error while moving downloaded file : "+filename);
                        if(typeof(callback) !== "undefined"){
                            callback(false,url,params);
                        }
                        throw err;
                    }else{
                        if(typeof(callback) !== "undefined"){
                            callback(true,url,params);
                        }
                    }
                });
              }
            },params);
          }
        });
    };

    this.unzipFile = function(filename,destFolder,callback,params){
      try{
        var zip_file = new self.zip(filename);
        /*
        var zipEntries = zip_file.getEntries(); // an array of ZipEntry records
        var total = zipEntries.length;
        var count = 0;
        zipEntries.forEach(function(zipEntry){
            count++;
            self.log("Extracting "+count+" / "+total);
            zip_file.extractEntryTo(zipEntry.entryName,destFolder,true,true);
        });
        */
        zip_file.extractAllTo(destFolder,true);

        if(typeof(callback) !== "undefined"){
            callback(true,filename,params);
        }
      }catch(e){
        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+10,self.name+": Function unzipFile() failed : error trying to unzip "+filename+" to "+destFolder);
        if(typeof(callback) !== "undefined"){
            callback(true,filename,params);
        }
      }
    };


    this.readDirectory = function(directory,callback,params){
        try{
            self.fs.readdir(directory, function(err,files){
                if(err){
                    self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+5,self.name+": Function readDirectory() failed : error while reading directory  : "+directory);
                    if(typeof(callback) !== "undefined"){
                        callback([],directory,params); // Send emtpy array if error
                    }
                    //throw err;
                }
                if(typeof(callback) !== "undefined"){
                    callback(files,directory,params);
                    //callback([],directory,params);
                }
            });

        }
        catch(e){
            if(typeof(callback) !== "undefined"){
              callback([],directory,params);
            }
        }
    };

    this.watchDirectory = function(directory,compare_files,callback,params){
        var compare_index = -1;
        var changed = [];
        self.fs.readdir(directory, function(err,files){
            if(err){
                self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+5,self.name+": Function readDirectory() failed : error while reading directory  : "+directory);
                if(typeof(callback) !== "undefined"){
                    callback(compare_files,[],params); // Send emtpy array if error
                }
                throw err;
            }
            for(var j=0;j<compare_files.length;j++){
                compare_index = files.indexOf(compare_files[j].filename);
                if(compare_index >= 0){
                    var stat = self.fs.statSync(directory+files[compare_index]);
                    var mdate = new Date(stat.ctime).getTime();
                    if(compare_files[j].mdate != mdate){
                        compare_files[j].mdate = new Date(stat.ctime).getTime();
                        changed.push(compare_files[j]);
                    }
                }else{
                    if(compare_files[j].mdate != 0){
                        compare_files[j].mdate = 0;
                        changed.push(compare_files[j]);
                    }
                }
            }

            if(typeof(callback) !== "undefined"){
                callback(compare_files,changed,params);
            }
        });
    };

    this.deleteRecursive = function(path){
        var files = [];
        if(self.fs.existsSync(path)) {
            if(self.fs.lstatSync(path).isDirectory()){
                files = self.fs.readdirSync(path);
                files.forEach(function(file,index){
                    var curPath = path + "/" + file;
                    if(self.fs.lstatSync(curPath).isDirectory()){
                        self.deleteRecursive(curPath);
                    } else { // delete file
                        try{
                          self.fs.unlinkSync(curPath);
                        }catch(e){
                          self.log("could not delete : "+curPath);
                        }
                    }
                });
                try{
                  self.fs.rmdirSync(path);
                }catch(e){
                  self.log("could not delete dir : "+path);
                }
            }else{
              try{
                self.fs.unlinkSync(path);
              }catch(e){
                self.log("could not delete : "+path);
              }
            }
        }
    };

    this.deleteFilesInDir = function(dir,filter,callback,params){
        self.fs.readdir(dir, function(err,files){
            if(err){
                self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+5,self.name+": Function deleteFilesInDir() failed : error while reading directory  : "+dir);
                if(typeof(callback) !== "undefined"){
                    callback([],dir,params); // Send emtpy array if error
                }
                throw err;
            }
            if(typeof(filter) === "undefined"){filter = "";}
            var deleted_files = [];
            for(var i=0;i<files.length;i++){
                if(filter != ""){
                    var index = files[i].lastIndexOf(filter);
                    if(index >= 0){
                        self.log(files[i]+" contains : "+filter+" delete");
                        self.fs.unlinkSync(self.rootDir+dir+files[i]);
                        deleted_files.push(files[i]);
                    }
                }else{
                    self.log("delete temp dir file : "+self.rootDir+dir+files[i]);
                    self.fs.unlinkSync(self.rootDir+dir+files[i]);
                    deleted_files.push(files[i]);
                }
            }

            if(typeof(callback) !== "undefined"){
                callback(deleted_files,dir,params);
            }
        });
    };

    this.deleteFile = function(filename,callback,params){
        self.deleteRecursive(filename);

        if(typeof(callback) !== "undefined"){
            callback(true,filename,params);
        }
    };

    this.moveFile = function(source,destination,callback,params){
        self.log("renaming : "+source+" to "+destination);
        self.fs.renameSync(source,destination);
        if(typeof(callback) !== "undefined"){
            callback(true,"",params);
        }
    };
    this.copyFile = function(source,destination,callback,params){
        self.log("copy : "+source+" to "+destination);
        self.fs.copyFileSync(source,destination, function(err){
            if(typeof(callback) !== "undefined"){
                callback(true,"",params);
            }
        });

    }

    this.copyRecursiveSync = function(src,dest){
      try{
        var exists = self.fs.existsSync(src);
        var stats = exists && self.fs.lstatSync(src);
        var isDirectory = exists && stats.isDirectory();
        if(exists && isDirectory){
          if(!self.fs.existsSync(dest)){
            self.fs.mkdirSync(dest);
          }
          self.fs.readdirSync(src).forEach(function(childItemName){
            self.copyRecursiveSync(self.path.join(src, childItemName),self.path.join(dest, childItemName));
          });
        }else{
          self.fs.writeFileSync(dest,self.fs.readFileSync(src));
        }
      }catch(e){
        self.log("Copy File : "+src+" to "+dest+" error");
      }
    }

    this.systemUpdate = function(update_folder,platform){
        self.log("Updating the system .... : "+platform);
        if(!self.systemIsUpdating){
            self.systemIsUpdating = true;
            var package_filename = "";
            if(platform == "WIN"){package_filename = "package_win.json";}
            if(platform == "MAC"){package_filename = "package_mac.json";}
            if(platform == "LINUX"){package_filename = "package_linux.json";}
            self.deleteFile(self.rootDir+"app_backup",function(){
              //  self.deleteFile(self.rootDir+"files_backup",function(){

                    try{
                        self.makeDir(self.rootDir+"files_backup",function(){
                           // self.makeDir(self.rootDir+"app_backup",function(){
                            self.moveFile(self.rootDir+self.ROUTES['MAIN_JS'].path,self.rootDir+"files_backup/"+self.ROUTES['MAIN_JS'].path,function(){
                                self.moveFile(self.rootDir+self.ROUTES['MAIN_HTML'].path,self.rootDir+"files_backup/"+self.ROUTES['MAIN_HTML'].path,function(){
                                    self.moveFile(self.rootDir+self.ROUTES['VER'].path,self.rootDir+"files_backup/"+self.ROUTES['VER'].path,function(){
                                        self.moveFile(self.rootDir+"app/",self.rootDir+"app_backup/",function(){
                                            try{
                                                self.moveFile(self.rootDir+update_folder+"files/app/",self.rootDir+"app/",function(){
                                                    self.moveFile(self.rootDir+update_folder+"files/"+self.ROUTES['MAIN_JS'].path,self.rootDir+self.ROUTES['MAIN_JS'].path);
                                                    self.moveFile(self.rootDir+update_folder+"files/"+self.ROUTES['MAIN_HTML'].path,self.rootDir+self.ROUTES['MAIN_HTML'].path);
                                                    self.moveFile(self.rootDir+update_folder+"files/"+self.ROUTES['VER'].path,self.rootDir+self.ROUTES['VER'].path);
                                                    //self.moveFile(self.rootDir+update_folder+"files/node_modules/",self.rootDir+"new_node_modules/");
                                                    if(!self.isARM){
                                                      self.moveFile(self.rootDir+update_folder+"files/"+package_filename,self.rootDir+"package.json");
                                                    }
                                                    self.moveFile(self.rootDir+update_folder+"files/watchdog.sh",self.rootDir+"watchdog.sh");
                                                    self.moveFile(self.rootDir+update_folder+"files/icon_manager.sh",self.rootDir+"icon_manager.sh");
                                                    if(platform == "LINUX"){
                                                      try{
                                                        require("child_process").exec("chmod +x "+self.rootDir+"watchdog.sh");
                                                        require("child_process").exec("chmod +x "+self.rootDir+"icon_manager.sh");
                                                      }catch(e){
                                                        console.log("Unable to set watchdog permissions");
                                                      }
                                                    }
                                                    self.deleteFile(self.rootDir+update_folder);
                                                });
                                            }
                                            catch(e){
                                                self.log(JSON.stringify(e));
                                                self.moveFile(self.rootDir+"app_backup/", self.rootDir+"app/",function(){})
                                                self.moveFile(self.rootDir+"files_backup/"+self.ROUTES['MAIN_JS'].path, self.rootDir+self.ROUTES['MAIN_JS'].path,function(){})
                                                self.moveFile(self.rootDir+"files_backup/"+self.ROUTES['MAIN_HTML'].path, self.rootDir+self.ROUTES['MAIN_HTML'].path,function(){})
                                                self.moveFile(self.rootDir+"files_backup/"+self.ROUTES['VER'].path, self.rootDir+self.ROUTES['VER'].path,function(){})
                                                /*
                                                if(self.fs.existsSync(self.rootDir+"new_node_modules/")){
                                                    self.deleteFile(self.rootDir+"new_node_modules/",function(){})
                                                }
                                                */
                                            }
                                        });
                                    })
                                })
                            })
                        })
                      //})
                    }
                    catch(e){
                        self.log(JSON.stringify(e));
                        try{
                            self.moveFile(self.rootDir+"app/",self.rootDir+"app_backup/",function(){
                                self.moveFile(self.rootDir+update_folder+"files/app/",self.rootDir+"app/",function(){
                                    self.moveFile(self.rootDir+update_folder+"files/"+self.ROUTES['MAIN_JS'].path,self.rootDir+self.ROUTES['MAIN_JS'].path);
                                    self.moveFile(self.rootDir+update_folder+"files/"+self.ROUTES['MAIN_HTML'].path,self.rootDir+self.ROUTES['MAIN_HTML'].path);
                                    self.moveFile(self.rootDir+update_folder+"files/"+self.ROUTES['VER'].path,self.rootDir+self.ROUTES['VER'].path);
                                    //self.moveFile(self.rootDir+update_folder+"files/node_modules/",self.rootDir+"new_node_modules/");
                                    self.deleteFile(self.rootDir+update_folder);
                                });
                            });
                        }
                        catch(ee){
                            self.log(JSON.stringify(ee));
                            try{
                                self.moveFile(self.rootDir+"files_backup/"+self.ROUTES['MAIN_JS'].path, self.rootDir+self.ROUTES['MAIN_JS'].path,function(){})
                                self.moveFile(self.rootDir+"files_backup/"+self.ROUTES['MAIN_HTML'].path, self.rootDir+self.ROUTES['MAIN_HTML'].path, function(){})
                                self.moveFile(self.rootDir+"files_backup/"+self.ROUTES['VER'].path, self.rootDir+self.ROUTES['VER'].path, function(){})
                            }
                            catch(eee){
                                self.log(JSON.stringify(eee));
                            }
                        }
                    }
               // })
            });
            self.reload(platform);
            self.systemIsUpdating = false;
        }
    };

    this.systemRepairUpdate = function(){
      // TODO
    };

    this.takeScreenshot = function(callback,params){
        var size = self.getScreenSize();
        var width = 250;
        var height = 140;
        if(size.height > size.width){
          width = 140;
          height = 250;
        }
        if(params){
            self.screenshooter("capture.jpg",{width: width, height: height, quality: 75}, function(data) {
                var img = new Image();
                img.crossOrigin = 'Anonymous';
                img.onload = function(){
                    var base64 = false;
                    var canvas = document.createElement('CANVAS');
                    var context = canvas.getContext('2d');
                    //250x140 Screenshot size
                    var width = 250;
                    var height = 140;
                    if (img.height > img.width){
                        width = 140;
                        height = 250;
                    }
                    canvas.height = height;
                    canvas.width = width;
                    //console.debug("takeScreenshot: width:"+ width+", height:"+ height+ "quality: "+params)
                    context.drawImage(img, 0, 0,width,height);
                    base64 = canvas.toDataURL("image/jpeg", params);
                    if(typeof(callback) !== "undefined"){
                        self.screenshotServer = base64.substring(23);
                        callback(base64.substring(23),params);
                    }
                    canvas = null;
                };
                var d = new Date();
                if(self.fs.existsSync("./capture.jpg")){
                  img.src = "capture.jpg?time="+d;
                }else{
                  img.src = "capture.png?time="+d;
                }
            });
        }
        else{
            self.screenshooter("capture.jpg",{width: width, height: height, quality: 75}, function(data) {
                var img = new Image();
                img.crossOrigin = 'Anonymous';
                img.onload = function(){
                    var base64 = false;
                    var canvas = document.createElement('CANVAS');
                    var context = canvas.getContext('2d');
                    //250x140 Screenshot size
                    var width = 250;
                    var height = 140;
                    if (img.height > img.width){
                        width = 140;
                        height = 250;
                    }
                    canvas.height = height;
                    canvas.width = width;
                    //console.debug("takeScreenshot: width:"+ width+", height:"+ height+ "quality: "+params)
                    context.drawImage(img, 0, 0,width,height);
                    base64 = canvas.toDataURL("image/jpeg", 1.0);
                    if(typeof(callback) !== "undefined"){
                        self.screenshotServer = base64.substring(23);
                        callback(base64.substring(23),params);
                    }
                    canvas = null;
                };
                var d = new Date();
                if(self.fs.existsSync("./capture.jpg")){
                  img.src = "capture.jpg?time="+d;
                }else{
                  img.src = "capture.png?time="+d;
                }
            });
        }
    };

    this.getScreenshot = function(callback){
        var size = self.getScreenSize();
        var width = 250;
        var height = 140;
        if(size.height > size.width){
          width = 140;
          height = 250;
        }
        self.screenshooter("capture.jpg",{width: width, height: height, quality: 75}, function(data) {
          var img = new Image();
          img.crossOrigin = 'Anonymous';
          img.onload = function(){
              var base64 = false;
              var canvas = document.createElement('CANVAS');
              var context = canvas.getContext('2d');
              //250x140 Screenshot size
              var width = 250;
              var height = 140;
              if (img.height > img.width){
                  width = 140;
                  height = 250;
              }
              canvas.height = height;
              canvas.width = width;
              context.drawImage(img, 0, 0,width,height);
              base64 = canvas.toDataURL("image/jpeg", 1.0);
              if(typeof(callback) !== "undefined"){
                  self.screenshotServer = base64.substring(23);
                  callback(base64.substring(23));
              }
              canvas = null;
          };
          var d = new Date();
          if(self.fs.existsSync("./capture.jpg")){
            img.src = "capture.jpg?time="+d;
          }else{
            img.src = "capture.png?time="+d;
          }
        });
    };

    this.getMousePos = function(){
      var pos = false;
      if(self.robot){
        pos = self.robot.getMousePos();
      }
      return pos;
    };

    this.setSystemDateTime = function(timestamp,callback,params){
        // TODO
    };

    this.rebootSystem = function(platform){
        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+6,self.name+": Reboot requested platform : "+platform);
        var child_process = require("child_process");

        if (platform == "LINUX")  {
            child_process.exec("reboot");
        }else if(platform == "MAC"){
            var reboot_msg = 'tell app "System Events" to restart';
            reboot_msg = "'"+reboot_msg+"'";
            child_process.exec("osascript -e "+reboot_msg);
        }else if(platform == "WIN"){
            child_process.exec("shutdown -t 0 -r -f");
        }

    };

    this.haltSystem = function(platform){
        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+6,self.name+": PowerOFF requested platform : "+platform);
        var child_process = require("child_process");

        if (platform == "LINUX")  {
            child_process.exec("poweroff");
        }else if(platform == "MAC"){
            var reboot_msg = 'tell app "System Events" to shut down';
            reboot_msg = "'"+reboot_msg+"'";
            child_process.exec("osascript -e "+reboot_msg);
        }else if(platform == "WIN"){
            child_process.exec("shutdown -t 0 -s -f");
        }
    };

    this.internetConnected = function() {
        require('dns').lookup(self.api_url, function(err) {
          if(err && err.code == "ENOTFOUND") {
              self.internet_connection = false;
          }else{
             self.internet_connection = true;
          }
        });

        return self.internet_connection;
    };

    this.getNetworkInfo = function(callback,params){
        var network_info = {
            ip: "0.0.0.0",
            netmask : "0.0.0.0",
            gateway : "0.0.0.0"
        };
        var ifaces = require('os').networkInterfaces();
        for(var i in ifaces){
            if(typeof(ifaces[i]) !== "undefined"){
              for(var j = 0; j < ifaces[i].length;j++){
                if(typeof(ifaces[i][j].family) !== "undefined"){
                  if(ifaces[i][j].family == "IPv4"){
                    if(ifaces[i][j].address != "127.0.0.1"){
                      //console.log(ifaces[i][j]);
                      network_info.ip = ifaces[i][j].address;
                    }
                  }
                }
              }
            }
        }
        if(typeof(callback) !== "undefined"){
            callback(network_info,params);
        }
        return network_info;
    };

    this.getScreens = function(){
        if(!self.screens){
          self.screens
            self.gui.Screen.Init();
            self.screens = self.gui.Screen.screens;
        }
        return self.screens;
    };

    this.getScreenSize = function(){
        self.gui.Screen.Init();

        var screens = self.gui.Screen.screens;
        var width = 0;
        var height = 0;

        for(var i=0;i<screens.length;i++){
            var screen_width = screens[i].bounds.x + screens[i].bounds.width;
            if(screen_width > width){
                width = screen_width;
            }
            var screen_height = screens[i].bounds.y + screens[i].bounds.height;
            if(screen_height > height){
                height = screen_height;
            }
        }
        //self.log("Screen TOTAL SIZE : "+width+"x"+height);

        return {'width':width,'height':height};
    };

    this.setMaxWindowDesktop = function(screen){
      if(self.platform == "LINUX"){
        try{
          require("child_process").exec("xset dpms 0 0 0  &> /dev/null; xset -dpms  &> /dev/null; xset dpms force on  &> /dev/null; xset s noblank &> /dev/null; xset s off &> /dev/null;");
        }catch(e){
          console.log("Unable to set DPMS Display Power Managemnet Settings on Linux");
        }
      }
      if(self.window_showing){
        var desktop = self.getScreenSize();
        if(typeof(screen) !== "undefined"){
          //if(parseInt(screen.w) != self.gui.Window.get().width || parseInt(screen.h) != self.gui.Window.get().height || parseInt(screen.x) != self.gui.Window.get().x || parseInt(screen.y) || self.gui.Window.get().y){
            self.gui.Window.get().leaveKioskMode();
            self.gui.Window.get().leaveFullscreen();
            // REAL WINDOW SIZE AND POSITION
            self.gui.Window.get().x = parseInt(screen.x);
            self.gui.Window.get().y = parseInt(screen.y);
            self.gui.Window.get().width = parseInt(screen.w);
            self.gui.Window.get().height = parseInt(screen.h);
            self.gui.Window.get().resizeTo(parseInt(screen.w),parseInt(screen.h));
          //}
        }else{
          //if(desktop.width != self.gui.Window.get().width || desktop.height != self.gui.Window.get().height){
            if(self.gui.Screen.screens.length <= 1){
              self.gui.Window.get().enterFullscreen();
            }else{
              self.gui.Window.get().leaveKioskMode();
              self.gui.Window.get().leaveFullscreen();
            }
            self.gui.Window.get().x = 0;
            self.gui.Window.get().y = 0;
            self.gui.Window.get().width = desktop.width;
            self.gui.Window.get().height = desktop.height;
            self.gui.Window.get().resizeTo(desktop.width,desktop.height);
          //}
        }
      }
    };

    this.getProxyForURL = function(url){
        return self.gui.App.getProxyForURL(url);
    };

    this.setProxyConfig = function(config){
    // Parses the rules from a string, indicating which proxies to use.
    //
    //   proxy-uri = [<proxy-scheme>"://"]<proxy-host>[":"<proxy-port>]
    //
    //   proxy-uri-list = <proxy-uri>[","<proxy-uri-list>]
    //
    //   url-scheme = "http" | "https" | "ftp" | "socks"
    //
    //   scheme-proxies = [<url-scheme>"="]<proxy-uri-list>
    //
    //   proxy-rules = scheme-proxies[";"<scheme-proxies>]
    //
    // Thus, the proxy-rules string should be a semicolon-separated list of
    // ordered proxies that apply to a particular URL scheme. Unless specified,
    // the proxy scheme for proxy-uris is assumed to be http.
    //
    // Some special cases:
    //  * If the scheme is omitted from the first proxy list, that list applies
    //    to all URL schemes and subsequent lists are ignored.
    //  * If a scheme is omitted from any proxy list after a list where a scheme
    //    has been provided, the list without a scheme is ignored.
    //  * If the url-scheme is set to 'socks', that sets a fallback list that
    //    to all otherwise unspecified url-schemes, however the default proxy-
    //    scheme for proxy urls in the 'socks' list is understood to be
    //    socks4:// if unspecified.
    //
    // For example:
    //   "http=foopy:80;ftp=foopy2"  -- use HTTP proxy "foopy:80" for http://
    //                                  URLs, and HTTP proxy "foopy2:80" for
    //                                  ftp:// URLs.
    //   "foopy:80"                  -- use HTTP proxy "foopy:80" for all URLs.
    //   "foopy:80,bar,direct://"    -- use HTTP proxy "foopy:80" for all URLs,
    //                                  failing over to "bar" if "foopy:80" is
    //                                  unavailable, and after that using no
    //                                  proxy.
    //   "socks4://foopy"            -- use SOCKS v4 proxy "foopy:1080" for all
    //                                  URLs.
    //   "http=foop,socks5://bar.com -- use HTTP proxy "foopy" for http URLs,
    //                                  and fail over to the SOCKS5 proxy
    //                                  "bar.com" if "foop" is unavailable.
    //   "http=foopy,direct://       -- use HTTP proxy "foopy" for http URLs,
    //                                  and use no proxy if "foopy" is
    //                                  unavailable.
    //   "http=foopy;socks=foopy2   --  use HTTP proxy "foopy" for http URLs,
    //                                  and use socks4://foopy2 for all other
    //                                  URLs.

    };

    this.getAddWhiteList = function(srcOrigin, destProtocol, destHost, allowDestinationSubdomains){
        self.gui.App.addOriginAccessWhitelistEntry(srcOrigin, destProtocol, destHost, allowDestinationSubdomains);
    };

    this.getRemoveWhiteList = function(srcOrigin, destProtocol, destHost, allowDestinationSubdomains){
        self.gui.App.removeOriginAccessWhitelistEntry(srcOrigin, destProtocol, destHost, allowDestinationSubdomains);
    };

    this.getDataPath = function(){
        return self.gui.App.dataPath;
    };

    this.setCrashDumpPath = function(folderPath){
        self.gui.App.setCrashDumpDir(folderPath);
        // Use self.gui.App.crashBrowser(), self.gui.App.crashRenderer() for testing.
    };

    this.clearCache = function(){
        self.gui.App.clearCache();
    };

    this.quit = function(){
        //var window = self.gui.Window.get();
        // SafeMode
        self.gui.Window.get().leaveKioskMode();
        self.gui.Window.get().leaveFullscreen();
        self.gui.Window.get().hide();
        self.window_showing = false;
        //self.togglePlayerMenu();
        //self.gui.App.closeAllWindows();
        // alternatively use  UnsafeMode
        //self.gui.App.quit();
    };

    this.reload = function(platform){
        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+7,self.name+": Relaunching App on platform : "+platform);
        self.log("Relaunching .... : "+platform);
        var child,child_process = require("child_process");
        if(platform == "MAC"){
            child = child_process.spawn("open", ["-n", "-a", self.process.execPath.match(/^([^\0]+?\.app)\//)[1]], {detached:true});
            child.unref();
        }else if(platform == "LINUX"){
            child = child_process.exec("admira_manager stop;sleep 1;admira_manager stop;sleep 1; admira_manager start&");
        }else if(platform == "WIN"){
            // TODO
            child = child_process.spawn(process.execPath, [], {detached: true});
        }
        self.gui.App.quit();
    };

    this.recover = function(){
        if(!self.player_paused_by_user){
            //var window = self.gui.Window.get();
            // SafeMode
            if(self.gui.Window.get().isAlwaysOnTop || !self.window_showing){
              self.gui.Window.get().show();
            }
            //self.gui.Window.get().enterFullscreen();
            //self.gui.Window.get().enterKioskMode();
            self.window_showing = true;
        }
    };

    this.vanish = function(){
      //var window = self.gui.Window.get();
      self.gui.Window.get().leaveKioskMode();
      self.gui.Window.get().leaveFullscreen();
      self.gui.Window.get().hide();
      self.window_showing = false;
    };

    this.setWindow = function(x,y,w,h){

        /* DEPRECATED
        var window = self.gui.Window.get();
        // See: https://github.com/nwjs/nw.js/wiki/Window for all properties
        window.x = x;
        window.y = y;
        window.width = w;
        window.height = h;
        */
    };

    this.alwaysOnTop = function(value){
        //var window = self.gui.Window.get();
        if(self.gui.Window.get().isAlwaysOnTop != value){
          self.gui.Window.get().setAlwaysOnTop(value);
        }
    };

    this.focusWindow = function(){
        //var window = self.gui.Window.get();
        //self.window.show();
        if(self.window_showing){
          self.gui.Window.get().focus();
          if(!self.gui.Window.get().isAlwaysOnTop){
            self.gui.Window.get().setAlwaysOnTop(true)
          }
          //self.gui.Window.get().requestAttention(true);
        }
        //self.window.moveTo(0,0);
    };

    this.disableRemoteControl = function(platform){
        self.deactivateControl();
        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+8,self.name+": Disable remote control on platform : "+platform);
        if (platform == "LINUX")  {
            require("child_process").exec("killall ssh; killall x11vnc;");
        }else if(platform == "MAC"){
            exec_command = "";
        }else if(platform == "WIN"){
            require("child_process").exec("taskkill.exe /F /IM plink.exe");
            require("child_process").exec("taskkill.exe /F /IM admiravnc.exe");
        }

    };

    this.enableRemoteControl = function(platform,tunnels){
        self.activateControl();
        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+8,self.name+": Enable remote control on platform : "+platform);
        if (platform == "LINUX")  {
          try{
            var installed_user = self.fs.readFileSync("installer_user");
            require("child_process").exec("killall ssh; chown "+installed_user+":"+installed_user+" id_rsa.ssh;sudo -u "+installed_user+" chmod 600 id_rsa.ssh; sudo -u "+installed_user+" ssh -i id_rsa.ssh -oStrictHostKeyChecking=no -oUserKnownHostsFile=/dev/null -N "+tunnels+"&");
            require("child_process").exec("killall x11vnc; sudo -u "+installed_user+" x11vnc -forever -shared -passwd admira -autoport 5959 &");
          }catch(e){
              self.log("Unable to start remote control");
          }
        }else if(platform == "MAC"){
            try{
              tunnels = tunnels.replace("5959","5900");
              require("child_process").exec("killall ssh; ssh -i id_rsa.ssh -oStrictHostKeyChecking=no -oUserKnownHostsFile=/dev/null -N "+tunnels);
            }catch(e){
                self.log("Unable to start remote control");
            }
        }else if(platform == "WIN"){
            try{
              require("child_process").exec("echo y | plink -ssh -i id_rsa.ppk -N "+tunnels);
              require("child_process").exec("admiravnc.exe -run");
            }catch(e){
                self.log("Unable to start remote control");
            }
        }
    };

    this.keepAwake = function(platform){
        var exec_command = "";
        var child_process = require("child_process");
        child_process.exec(exec_command);
        if (platform == "LINUX")  {
            exec_command = "xset dpms 0 0 0  &> /dev/null;xset -dpms  &> /dev/null;xset dpms force on  &> /dev/null;";
        }else if(platform == "MAC"){
            exec_command = "";
        }else if(platform == "WIN"){
            exec_command = "";
        }

        if(exec_command != ""){
             child_process.exec(exec_command);
        }
    };

    this.activateControl = function(){
      self.debug = true;
      self.SocketConnector.createSocket();
    };
    this.deactivateControl = function(){
      self.debug = false;
      self.SocketConnector.disconnectSocket();
    };
    this.sendControlLog = function(ori, name, text){
        if(typeof(self.SocketConnector) !== "undefined"){
          self.SocketConnector.logsSocket(ori, (new Date()).toTimeString().split(' ')[0] +" .. " + name+ "--"+text);
        }
    };
    this.changeCore = function(id){
        self.SocketConnector.changeCore(id);
    };
    this.saveIdContent = function(idContent){
        self.id_content_now = idContent;
    };
    this.get_tunnels_params = function(){
        self.api_connector.get(self.api_url,self.api_ver,"player/get_tunnels_params/id",self.core_id,self.remoteResponse);
    };
    this.getInfoSystem = function(){

    };


    // NOT IN USE
    this.socketServerClose = function(uuid,error_code,reason){
        // EXAMPLE
        // wsserver.close({'uuid':'8e176b14-a1af-70a7-3e3d-8b341977a16e'}, 4000, 'my reason');
        /*
        if(self.webwebsocket_server){
            self.websocket_server.close({'uuid':uuid},error_code,reason);
        }
        */
    };

    this.socketServerStop = function(){
        if(self.webwebsocket_server){
            self.websocket_server.close();
            self.websocket_active = false;
        }
    };

    this.socketServerSend = function(conn,msg){
        // TODO
    };

    this.socketServerBroadcast = function(msg){
        if(self.websocket_server){
            var served_clients = [];
            self.websocket_server.clients.forEach(function each(client){
                if(client.readyState === 1){
                    if(served_clients.indexOf(client._socket.remoteAddress) < 0){
                      served_clients.push(client._socket.remoteAddress);
                      client.send(JSON.stringify(msg),function ack(error){
                          // If error is not defined, the send has been completed, otherwise the error
                          // object will indicate what failed.
                          if(typeof(error) !== "undefined"){
                              self.log("Websocket server send error ... ");
                          }
                      });
                    }
                }
            });
        }
    };

    this.socketServerStart = function(port,onOpenCallback,onMessageCallback,onTerminatedCallback){
        if(typeof(port) !== "undefined"){self.websocket_port = port;}
        try{
            if(!self.websocket_server){
                self.websocket_server = new self.ws.Server({'port': self.websocket_port});
                self.websocket_active = true;
                self.websocket_server.on('connection', self.onSocketOpen);
                self.websocket_server.on('close', self.onSocketClose);
                self.websocket_server.on('error', self.onSocketError);
                //self.websocket_server.on('message', self.onSocketMessage);
            }
            if(typeof(onOpenCallback) !== "undefined"){
              self.websocket_onOpenCallback = onOpenCallback;
            }
            if(typeof(onMessageCallback) !== "undefined"){
             self.websocket_onMessageCallback = onMessageCallback;
            }
            if(typeof(onTerminatedCallback) !== "undefined"){
             self.websocket_onTerminatedCallback = onTerminatedCallback;
            }


        }catch(e){
            self.websocket_active = false;
            self.onSocketFailure(e);
        }

    };

    this.onSocketFailure = function(error){
        self.log('Stopped ');
    };

    this.onSocketOpen = function(conn){
        self.log('onSocketOpen A user connected');
        conn.on("message",self.onSocketMessage);
    };

    this.onSocketMessage = function(msg){
        self.log("onSocketMessage : Message from client !!! ");
        self.websocket_onMessageCallback(msg);
    };

    this.onSocketClose = function(conn){
        self.log('onSocketClose A user disconnected from');
    };

    this.onSocketError = function(conn){
        self.log('onSocketError An error occurred ');
    };

    this.onSocketStart = function(addr, port){
        self.log('Listening on '+addr+':'+port);
    };

    this.onSocketStartFail = function(reason){
        self.log('Did not start.');
    };

    this.onSocketStop = function(addr, port){
        self.log('Stopped on '+addr+':'+port);
    };

    this.onSocketStopFail = function(reason){
        self.log('Did not stop');
    };

    this.setContentRotation = function(options, callback_setContentRotation) {
    }

    this.getDisplayOrientation = function(callback,params) {

    }

    /*socket*/
    this.getLIC = function(callback){
      self.readFile("admira.lic", function(resultR, file){
        if(resultR){
          callback("admira.lic = "+resultR);
        }
        else{
          callback("error read admira.lic");
        }
      });
    }
    this.createLIC = function(data, callback){
      /*self.writeFile("admira.lic", data.lic, function(resultR, file){
        if(resultR){
          callback("admira.lic ("+data.lic+") created");
        }
        else{
          callback("admira.lic not created");
        }
      });*/
      callback("no function");
    }
    this.deleteAll = function(callback){
      self.deleteFile("content/",function(result){
        self.deleteFile("xmls/",function(result){
          self.deleteFile("log/",function(result){
            self.deleteFile("core.xml",function(result){
              self.deleteFile("week.tstamp",function(result){
                setTimeout(function(){
                  self.rebootSystem();
                },2000);
                if(err){
                  callback(JSON.stringify(err));
                }else{
                  callback(result);
                }
              });
            });
          });
        });
      });
    };

    this.deleteAllAll = function(callback){
      callback("no function");
    };
    this.getInfoSystemPlus = function(callback){
      callback("no function");
    };
    this.getTimers = function(callback){
      callback("no function");
    };
    this.clearCache = function(callback) {
      callback("no function");
    };
    this.calculateReboot = function(callback) {
      callback("no function");
    };
    this.copyUSB = function (callback){
      callback("no function");
    };
    this.deleteAll2 =  function(callback){
      callback("no function");
    };
    this.upgradeApplication = function (callback_upgradeApplication){
      callback_upgradeApplication("no function");
    };
    this.upgradeApplicationInsert = function(url, callback_upgradeApplicationInsert){
      if(typeof(url) !== "undefined"){
        var params = {'filename':"app.exe",'platform':self.platform};    
        if(self.platform != "LINUX"){
          self.fileExists(self.rootDir+self.ROUTES['TMP'].path+params.filename,function(exists){
            if(!exists){
              if(!self.systemIsUpdating){
                self.systemIsUpdating = true;
                self.downloadFile(url,"./",function(ok,url,params){
                  if(ok){
                    self.log("App Upgrade for on : "+self.platform);
                    var child_process = require("child_process");
                    if (self.platform == "MAC")  {
                        child_process.exec("chmod +x "+params.filename+"; ./"+params.filename+"&");
                    }else if(self.platform == "WIN"){
                        self.log("Executing Runtime Upgrade for on : "+self.platform);
                        child_process.exec(""+params.filename);
                    }
                    self.systemIsUpdating = false;
                  }
                  callback_upgradeApplicationInsert(ok);
                },params);
              }else{
                callback_upgradeApplicationInsert("Already Upgrading APP ... ");
              }
            }else{
              callback_upgradeApplicationInsert("Already Downloading APP ... ");
            }
          });
        }else{
          callback_upgradeApplicationInsert("Unavailable ... Application Insert on Linux depends on Architecture, please upgrade runtime and app separetly");
        }
      }else{
        callback_upgradeApplicationInsert("no url on data");
      }
    }
    this.upgradeApplicationInstall = function(callback_upgradeApplicationInstall){
        callback_upgradeApplicationInstall("no function");
    }
    this.upgradeApplicationNow = function(callback_upgradeApplicationNow){
        callback_upgradeApplicationNow("no function");
    }
    this.updateApplicationS = function(callback_updateApplicationS){
        callback_updateApplicationS("no function");
    }


    this.screenControllersRelation = {
        7: 'PANASONIC_RS232',
        //8	: 'LG_RS232',
        11: 'ALBIRAL_RS232',          // Digiview
        15: 'RLY8_RS232',
        27: 'PHILIPS_RS232',
        56: 'PCB_RS232',
        62: 'LG_IMPACT_RS232' ,
        44: 'SAMSUNG_RJ45' ,
        64: 'DYNASCAN_RS232' ,
        24: 'SAMSUNG_RS232' ,
        55: 'EPSON_RS232',
        61: 'LG_WINDOWS_HDMI',
        00: ''
    };


    this.generateNewScreenController = function ( screenData, exceptionList, counter){
        var controller = false;
        var controllerName = self.screenControllersRelation[screenData.model]

        if ( typeof controller != 'undefined' ){
            try {
                eval(fs.readFileSync('./app/screenControllers/' + controllerName + '.js')+'');
                var controller = new Screen();
            }
            catch(error) {
                console.log (error);
                return false;
            }
            if  ( controller.init( screenData, exceptionList, self.platform, self.architecture, counter) ) {
                return controller;
            }
            return false;
        }else {
            return false;
        }
    }

    this.updateFirmware = function(data, callback_updateFirmware){
      if(typeof(data.url) !== "undefined"){
        var params = {'filename':"runtime.exe"};    
        var nw_filename = "";
        var is_transfer = false;
        try{
          nw_filename = nw.__filename.substring(nw.__filename.lastIndexOf("/")+1,nw.__filename.length);
          if(nw_filename !== "admira.html"){
            is_transfer = true;
          }
          self.log("Runtime Upgrade Request on : "+nw_filename+" is_transfer : "+is_transfer);
        }catch(e){
          callback_updateFirmware("Error on updateFirmware");
        }
        if(!is_transfer){
            if(!self.systemIsUpdating){
              self.systemIsUpdating = true;
              var url = data.url;
              var arch = "";
              try{
                if(self.platform == "LINUX"){
                  if(self.architecture == "x86_64"){
                    arch = "x64";
                  }else if(self.architecture == "armv7l" || self.architecture == "armv8l"){
                    arch = "ARM";
                  }else if(self.architecture == "i686" || self.architecture == "i586" || self.architecture == "i486" || self.architecture == "i386"){
                    arch = "x32";
                  }
                  if(data.arch !== "undefined"){
                    if(arch == data.arch.name){
                      url = data.arch.url;
                    }else{
                      url ="";
                    }
                  }
                }
              }catch(e){
                console.log("No Arch on data");
              }
              if(url != ""){
                self.downloadFile(url,"./",function(ok,url,params){
                  if(ok){
                    self.log("Runtime Upgrade on : "+self.platform);
                    var execute_command = "";
                    if (self.platform == "LINUX")  {
                        execute_command = "chmod +x "+params.filename+"; ./"+params.filename+"&";
                    }else if(self.platform == "MAC"){
                        execute_command = "chmod +x "+params.filename+"; ./"+params.filename+"&";
                    }else if(self.platform == "WIN"){
                        self.log("Executing Runtime Upgrade for on : "+self.platform);
                        execute_command = ""+params.filename;
                    }
                    setTimeout(function(){
                      try{
                        self.log("Executing Delayed Runtime");
                        var child_process = require("child_process");
                        require("child_process").exec(execute_command);
                      }catch(e){
                        self.log("Executing Runtime Exception");
                        console.log(e);
                      }
                    },5000);
                    callback_updateFirmware(ok);
                  }else{
                    callback_updateFirmware(ok);
                    self.systemIsUpdating = false;
                  }
                },params);
              }else{
                callback_updateFirmware("Somthing went wrong ... bad architecture ? ARCH : "+arch+" URL : "+url);
                self.systemIsUpdating = false;
              }
              
            }else{
              callback_updateFirmware("Already Upgrading firmware ... ");
              self.systemIsUpdating = false;
            }
        }else{
          callback_updateFirmware("Already Downloading firmware ... ");
          self.systemIsUpdating = false;
        }
      }else{
        callback_updateFirmware("no url on data");
        self.systemIsUpdating = false;
      }
    };
    this.getStatusPanel = function(url, callback_getStatusPanel){
      callback_getStatusPanel("no function");
    };
    this.setPanelOff = function(url, callback_setPanelOff){
      if(typeof(callback_setPanelOff) != "undefined"){
        callback_setPanelOff("Panel Off by suspend system android");
      }
    };
    this.setPanelOn = function(url, callback_setPanelOn){
      if(typeof(callback_setPanelOn) != "undefined"){
        callback_setPanelOn("no function");
      }
    };
    this.changePictureMode = function(callback_changePictureMode){
        callback_changePictureMode("no function");
    }
    this.getPictureProperty = function(callback_getPictureProperty){
        callback_getPictureProperty("no function");
    }
    this.setPortraitMode = function(data, callback_setPortraitMode){
        callback_setPortraitMode("no function");
    }
    this.getContentRotation = function(callback_getContentRotation){
        callback_getContentRotation("no function");
    }
    this.setContentRotation = function(data, callback_setContentRotation){
        callback_setContentRotation("no function");
    }
    this.getVideoStatus = function(callback_getVideoStatus){
        callback_getVideoStatus("no function");
    }
    this.shellExec = function(command,callback,params){
      var result = "";
      if(typeof(callback) !== "undefined"){
          callback(result,command,params);
      }
    };

    this.findSyncIp = function (callback){
        /*
        window.RTCPeerConnection = window.RTCPeerConnection || window.mozRTCPeerConnection || window.webkitRTCPeerConnection;  
        var pc = new RTCPeerConnection({iceServers:[]}), noop = function(){};      
        pc.createDataChannel("");  
        pc.createOffer(pc.setLocalDescription.bind(pc), noop);
        pc.onicecandidate = function(ice){  //listen for candidate events
            if(!ice || !ice.candidate || !ice.candidate.candidate)  return;
            var candidate = /([0-9]{1,3}(\.[0-9]{1,3}){3}|[a-f0-9]{1,4}(:[a-f0-9]{1,4}){7})/.exec(ice.candidate.candidate);
            if(typeof(candidate) != "undefined" && candidate != null){
              if(typeof(candidate[1]) != "undefined"){
                var ip = candidate[1];
                pc.onicecandidate = noop;
                if(typeof(callback) !== "undefined"){
                  callback(ip);
                }
              }
            }
        };
        */
        var info = self.getNetworkInfo();
        if(typeof(callback) !== "undefined"){
          callback(info.ip);
        }
    };
    this.getPowerOnOffHistory = function(callback_getPowerOnOffHistory){
        callback_getPowerOnOffHistory("no function");
    }
    this.getBrigthness = function(callback_getBrigthness){
        callback_getBrigthness("no function");
    }
    this.setBrightness = function(brightness, callback_setBrightness){
        callback_setBrightness("no function");
    }
    this.getContrast = function(callback_getContrast){
        callback_getContrast("no function");
    }
    this.setContrast = function(contrast, callback_setContrast){
        callback_setContrast("no function");
    }
    this.getBacklight = function(callback_getBacklight){
        callback_getBacklight("no function");
    }
    this.setBacklight = function(backlight, callback_setBacklight){
        callback_setBacklight("no function");
    }
    this.geConfigPanel = function(callback_geConfigPanel){
        callback_geConfigPanel("no function");
    }
    this.setConfigPanel = function(data, callback_setConfigPanel){
        callback_setConfigPanel("no function");
    }
    this.setNativePortraitMode = function(data, mode, callback_setNativePortraitMode){
        callback_setNativePortraitMode("no function");
    }
    this.getStatusRemoteControl = function(callback_getStatusRemoteControl){
      callback_getStatusRemoteControl("no function")
    }
    this.setStatusRemoteControl = function(data, callback_setStatusRemoteControl){
        callback_setStatusRemoteControl("no function");
    }
}
NwjsSystemController.prototype = new SystemController();
