var execFile = require('child_process').execFile;
var fs = require('fs');

function HDMI_driverController(){

  if(arguments.callee._singletonInstance){
    return arguments.callee._singletonInstance;
  }

  arguments.callee._singletonInstance = this;

  var self = this; self.name = "HDMI_driverController"; // MANDATORY

  this.init = function(platform, architecture){
  }

  this.queue = [];
  this.busy = false;
  this.step = 0;
  this.currentAction = null;

  this.checkTstamp = new Date().getTime();

  this.validationTimer = null;
  this.validationTimeout = 2000;

  this.writeTimer = null;
  this.writeTimeout = 1000;

  this.processQueueTimer = null;
  this.processQueueTimeout = 500;

  this.processStepTimer = null;
  this.processStepTimeout = 500;

  this.processCommandTimer = null;
  this.processCommandTimeout = 2000;

  this.resetCounter = 0;
  this.resetCounterMax = 5;
  this.resetTimeout = 5000;

  /*********************************** ACCESS FUNCTIONS ***********************************/

  this.send = function(command, port, callback){
    self.queue.push([[command, port, callback]]);
    if(self.busy) return;
    self.busy = true;
    processQueue();
  }

  this.sendArray = function(commandArray, portArray, callbackArray){
    var finalCommand = [];
    for(var i = 0; i < commandArray.length; i++){
      finalCommand[i] = [commandArray[i], portArray[i], callbackArray[i]];
    }
    self.queue.push(finalCommand);
    if(self.busy) return;
    self.busy = true;
    processQueue();
  }

  this.sendPriority = function(command, port, callback){
    self.queue.unshift([[command, port, callback]]);
    if(self.busy) return;
    self.busy = true;
    processQueue();
  }

  this.sendPriorityArray = function(commandArray, portArray, callbackArray){
    var finalCommand = [];
    for(var i = 0; i < commandArray.length; i++){
      finalCommand[i] = [commandArray[i], portArray[i], callbackArray[i]];
    }
    self.queue.unshift(finalCommand);
    if(self.busy) return;
    self.busy = true;
    processQueue();
  }

  // Private functions can exist in the closure too; they can see the state but
  // outsiders can't touch them

  //function private() { ... }
  /*********************************** QUEUE FUNCTION ***********************************/

  function processQueue(){
  	self.step = 0;
    var next = self.queue.shift();
    if(!next){
      self.busy = false;
      return;
    }
    self.currentAction = next;
    processCommand();
  }

  function processStep(){
    self.step++;
    processCommand();
  }

  var last_status = "Disconnected";

var shutdownInterval;  function processCommand(){
console.log("processCommand()");
    self.checkTstamp = new Date().getTime();
    clearTimeouts();
    if(self.currentAction[self.step][0]){
      try{
        var monitorPowerPath = __dirname + "\\app\\screenDrivers\\HDMI\\MonitorPower.exe";
          if(self.currentAction[self.step][0].split(" ")[0] == "-p"){
	           if(self.currentAction[self.step][0].split(" ")[1] == "On"){
console.log("-p On");
               if(shutdownInterval){clearInterval(shutdownInterval);}
               last_status = "On";
               execFile(monitorPowerPath, self.currentAction[self.step][0].split(" "), (err, stdout, stderr) => {
                 if(err){
                   clearTimeouts();
                   self.validationTimer = setTimeout(validationFail, self.validationTimeout);
                   self.currentAction[self.step][2](err, null, self.checkTstamp, validation);
                 }else{
                   var data = stdout;
                   try{
                     clearTimeouts();
                     self.validationTimer = setTimeout(validationFail, self.validationTimeout);
                     self.currentAction[self.step][2](null, last_status, self.checkTstamp, validation);
                   }catch(err){
                   }
                 }
               });
            }else{
console.log("-p Off");
              last_status = "Off";
              shutdownInterval = setInterval(execFile, 30000, monitorPowerPath, self.currentAction[self.step][0].split(" "), (err, stdout, stderr) => {
                if(err){
                  clearTimeouts();
                  self.validationTimer = setTimeout(validationFail, self.validationTimeout);
                  self.currentAction[self.step][2](err, null, self.checkTstamp, validation);
                }else{
                  var data = stdout;
                  try{
                    clearTimeouts();
                    self.validationTimer = setTimeout(validationFail, self.validationTimeout);
                    self.currentAction[self.step][2](null, last_status, self.checkTstamp, validation);
                  }catch(err){
console.log(err);
                  }
                }
              });
	    }
          }else{
console.log("-q");
	           clearTimeouts();
	           self.validationTimer = setTimeout(validationFail, self.validationTimeout);
	           self.currentAction[self.step][2](null, last_status, self.checkTstamp, validation);
          }
      }catch(err){console.log(err);}
    }else{
      clearTimeouts();
      self.validationTimer = setTimeout(validationFail, self.validationTimeout);
      self.currentAction[self.step][2](err, null, self.checkTstamp, validation);
    }
  }


  /*********************************** RESPONSE FUNCTIONS ***********************************/

  function validation(action, sendCheckTstamp, callback){
    if(sendCheckTstamp == self.checkTstamp){
      if(action == "next"){
        self.checkTstamp = new Date().getTime();
        self.writeRetry = 0;
        clearTimeouts();
        self.processQueueTimer = setTimeout(processQueue, self.processQueueTimeout);
      }else if(action == "step"){
        self.checkTstamp = new Date().getTime();
        self.writeRetry = 0;
        clearTimeouts();
        self.processStepTimer = setTimeout(processStep, self.processStepTimeout);
      }else if(action == "retry"){
        self.checkTstamp = new Date().getTime();
        self.writeRetry = 0;
        clearTimeouts();
        self.processCommandTimer = setTimeout(processCommand, self.processCommandTimeout);
      }else{
        clearTimeouts();
        validationFail();
      }
    }else{
      if(action != "next"){
        callback();
      }
    }
  }

	function validationFail(){
  	self.checkTstamp = new Date().getTime();
  	self.writeRetry = 0;
  	clearTimeouts();
  	self.processQueueTimer = setTimeout(processQueue, self.processQueueTimeout);
  }

	function clearTimeouts(){
  	clearTimeout(self.validationTimer);
  	clearTimeout(self.writeTimer);
  	clearTimeout(self.processQueueTimer);
  	clearTimeout(self.processStepTimer);
  	clearTimeout(self.processCommandTimer);
  	return;
  }

  /*********************************** CACHE FUNCTIONS ***********************************/
  /**
  * Removes a module from the cache
  */
  function purgeCache(moduleName) {
    // Traverse the cache looking for the files
    // loaded by the specified module name
    searchCache(moduleName, function (mod) {
        delete require.cache[mod.id];
    });

    // Remove cached paths to the module.
    // Thanks to @bentael for pointing this out.
    Object.keys(module.constructor._pathCache).forEach(function(cacheKey) {
      if (cacheKey.indexOf(moduleName)>0) {
        delete module.constructor._pathCache[cacheKey];
      }
    });
  };

  /**
  * Traverses the cache to search for all the cached
  * files of the specified module name
  */
  function searchCache(moduleName, callback) {
    // Resolve the module identified by the specified name
    var mod = require.resolve(moduleName);

    // Check if the module has been resolved and found within
    // the cache
    if (mod && ((mod = require.cache[mod]) !== undefined)) {
      // Recursively go over the results
      (function traverse(mod) {
        // Go over each of the module's children and
        // traverse them
        mod.children.forEach(function (child) {
          traverse(child);
        });

        // Call the specified callback providing the
        // found cached module
        callback(mod);
      }(mod));
    }
  };
}
